OccupiedEntry

Enum OccupiedEntry 

pub enum OccupiedEntry<'a, P: JointPrefix, T> {
    P1(OccupiedEntry<'a, P::P1, T>),
    P2(OccupiedEntry<'a, P::P2, T>),
}
A mutable view into an occupied entry. An occupied entry represents a node that is already present on the tree.

Variants



P1(OccupiedEntry<'a, P::P1, T>)

Occupied entry for the first prefix variant



P2(OccupiedEntry<'a, P::P2, T>)

Occupied entry for the second prefix variant

Implementations

Source

impl<P: JointPrefix, T> OccupiedEntry<'_, P, T>

pub fn key(&self) -> P

Gets a reference to the key in the entry. This is the key that is currently stored, and not the key that was used in the insert.

pub fn get(&self) -> &T

Gets a reference to the value in the entry.

pub fn get_mut(&mut self) -> &mut T

Gets a mutable reference to the value in the entry.

Prefixes are not stored verbatim. They are reconstructed from the trie position, so host bits masked out by the prefix length are not preserved.

use prefix_trie::joint::map::Entry;
let mut pm: JointPrefixMap<ipnet::IpNet, _> = JointPrefixMap::new();
pm.insert("192.168.1.1/24".parse()?, 1);
if let Entry::Occupied(mut e) = pm.entry("192.168.1.0/24".parse()?) {
    *e.get_mut() += 1;
}
assert_eq!(
    pm.get_key_value(&"192.168.1.0/24".parse()?),
    Some(("192.168.1.0/24".parse()?, &2))
);
pub fn insert(self, value: T) -> T

Insert a new value into the entry, returning the old value.

use prefix_trie::joint::map::Entry;
let mut pm: JointPrefixMap<ipnet::IpNet, _> = JointPrefixMap::new();
pm.insert("192.168.1.1/24".parse()?, 1);
if let Entry::Occupied(mut e) = pm.entry("192.168.1.0/24".parse()?) {
    e.insert(2);
}
assert_eq!(
    pm.get_key_value(&"192.168.1.0/24".parse()?),
    Some(("192.168.1.0/24".parse()?, &2))
);
pub fn remove(self) -> T

Remove the current value and return it. Empty trie nodes may be left in place (the same effect as JointPrefixMap::remove_keep_tree).